@@ -4,6 +4,7 @@ import pymc as pm
 from bn_testing.transformations import (
     Linear,
     Polynomial,
+    Constant,
 )
 
 
@@ -26,7 +27,7 @@ class Conditional(object):
         self.random = random
 
     def make_noise(self):
-        return pm.Normal.dist(mu=0, sigma=0.1)
+        return pm.Normal.dist(mu=0, sigma=0.05)
 
     def make_source(self):
         return pm.Beta.dist(
@@ -61,10 +62,11 @@ class PolynomialConditional(Conditional):
     Conditional that builds polynomial transformations
     """
 
-    def __init__(self, min_terms=1, max_terms=5, max_degree_add=10):
+    def __init__(self, min_terms=1, max_terms=5, max_degree_add=10, with_tanh=True):
         self.min_terms = min_terms
         self.max_terms = max_terms
         self.max_degree_add = max_degree_add
+        self.with_tanh = with_tanh
 
     def _get_random_exponent(self, degree, n_variables):
         """
@@ -101,4 +103,24 @@ class PolynomialConditional(Conditional):
 
         signs = self.random.choice([-1, 1], size=n_monomials)
         coefs = signs * self.random.uniform(1, 10, size=n_monomials)
-        return Polynomial(parents, exponents, coefs)
+        return Polynomial(
+            parents=parents,
+            exponents=exponents,
+            coefs=coefs,
+            with_tanh=self.with_tanh
+        )
+
+
+class ConstantConditional(Conditional):
+    """
+    A conditional yielding constant values. Used in intervensions and to compute causal effects.
+    """
+
+    def __init__(self, value):
+        self.value = value
+
+    def make_transformation(self, parents):
+        return Constant(parents, self.value)
+
+    def make_noise(self):
+        return pm.math.constant(0)
