@@ -6,7 +6,7 @@ import pymc as pm
 
 from abc import ABCMeta
 
-from bn_testing.helpers import sigmoid
+from bn_testing.conditionals import ConstantConditional
 
 logger = logging.getLogger(__name__)
 
@@ -18,13 +18,12 @@ class BayesianNetwork(metaclass=ABCMeta):
         We assume that the order of the nodes is a topological ordering of the resulting graph.
 
     Args:
-        n_nodes (int): Number of nodes
         p (float): Erdös-Renyi probability
         dag (bn_testing.dags.DAG): A DAG generation method
         conditionals (bn_testing.conditionals.Conditionals): A conditional type
     """
 
-    def __init__(self, dag=None, conditionals=None, n_nodes=None, random_state=None):
+    def __init__(self, dag=None, conditionals=None, random_state=None):
         self.random_state = random_state
         self.random = np.random.RandomState(self.random_state)
 
@@ -34,19 +33,19 @@ class BayesianNetwork(metaclass=ABCMeta):
         self.conditionals = conditionals
         self.conditionals.init(self.random)
 
-        self.n_nodes = n_nodes
-
         self.generate()
+        self.n_nodes = self.dag.number_of_nodes
 
     def generate(self):
         """
         Generates the mod
         """
         logger.info('Generate DAG')
-        self.dag = self.dag_gen.generate(self.n_nodes)
+        self.dag = self.dag_gen.generate()
         logger.info('Generate models')
         self.transformations = self._build_transformations()
         self.sources = self._build_sources()
+        self.noises = self._build_noises()
 
     @property
     def edges(self):
@@ -61,7 +60,8 @@ class BayesianNetwork(metaclass=ABCMeta):
             conditionals = self.conditionals
         else:
             conditionals.init(self.random)
-        self.transformations[node] = conditionals(self._get_parents(node))
+        self.transformations[node] = conditionals.make_transformation(self._get_parents(node))
+        self.noises[node] = conditionals.make_noise()
 
     @property
     def nodes(self):
@@ -70,6 +70,26 @@ class BayesianNetwork(metaclass=ABCMeta):
     def _get_parents(self, node):
         return [n for n, _ in self.dag.in_edges(node)]
 
+    def is_source(self, node):
+        """
+        Checks if the given node is a source node, i.e., has no parents.
+
+        Args:
+            node (str): Name of a node in the dag
+
+        Returns:
+            bool: True if node is a source, false otherwise
+        """
+        return len(self._get_parents(node)) == 0
+
+    def _build_noises(self):
+        noises = {}
+        for node in self.nodes:
+            parents = self._get_parents(node)
+            if len(parents) > 0:
+                noises[node] = self.conditionals.make_noise()
+        return noises
+
     def _build_sources(self):
         """
         Builds PyMC variables that correspond to source nodes in the DAG.
@@ -101,15 +121,51 @@ class BayesianNetwork(metaclass=ABCMeta):
 
         """
         if len(parents_mapping.keys()) > 0:
-            var = sigmoid(
-                self.transformations[node].apply(parents_mapping) + self.conditionals.make_noise()
-            )
+            var = self.transformations[node].apply(parents_mapping) + self.noises[node]
         else:
             # As generating the source introduces some randomness we would like to fix, we have to
             # build the source variables only once and reuse them here.
             var = self.sources[node]
         return var
 
+    def compute_average_causal_effect(self, node_from=None, node_onto=None, value=None, n=1000):
+        """
+        Computes the average causal effect of a node that has a certain value onto another node.
+
+        Args:
+            node_from (str): Name of node that gets the intervention
+            node_onto (st): Variable whose change should be computed
+            value (float): Value of intervention
+            n (int): Sample size to approximate the expected values
+
+        Returns:
+            float: The average causal effect
+        """
+        df_orig = self.sample(n=n, nodes=[node_onto])
+
+        if self.is_source(node_from):
+            source = self.sources[node_from]
+            transformation = ConstantConditional(value=value).make_transformation([])
+            self.sources[node_from] = transformation.apply({})
+        else:
+            transformation = self.transformations[node_from]
+            noise = self.noises[node_from]
+
+        self.modify_transformation(
+            node=node_from,
+            conditionals=ConstantConditional(value=value)
+        )
+
+        df_intervent = self.sample(n=n, nodes=[node_onto])
+
+        if self.is_source(node_from):
+            self.sources[node_from] = source
+        else:
+            self.transformations[node_from] = transformation
+            self.noises[node_from] = noise
+
+        return df_intervent[node_onto].mean() - df_orig[node_onto].mean()
+
     def _build_variables(self):
         variables = {}
 
@@ -117,10 +173,11 @@ class BayesianNetwork(metaclass=ABCMeta):
             parents_mapping = {
                 parent: variables[parent] for parent, _ in self.dag.in_edges(node)
             }
+
             variables[node] = self._build_variable(node, parents_mapping)
         return variables
 
-    def sample(self, n):
+    def sample(self, n, nodes=None):
         """
         Samples `n` many identic and independent observations from the Bayesian network.
 
@@ -133,14 +190,17 @@ class BayesianNetwork(metaclass=ABCMeta):
             rows
         """
 
+        if nodes is None:
+            nodes = self.nodes
+
         logger.info('Build variables')
         variables = self._build_variables()
 
         logger.info('Start sampling')
-        data = pm.draw([variables[n] for n in self.nodes], draws=n)
+        data = pm.draw([variables[n] for n in nodes], draws=n)
         df = pd.DataFrame(
             data=np.array(data).T,
-            columns=self.nodes
+            columns=nodes
         )
         return df
 
